unit MBar;

{-------------------------------------------------------------------------------

 TMenuBar

 MenuBar component - (c) 2004 Janssen Software - www.janssensoftware.com
 
 Useage:
   Place a TMenuBar on a form
   Assign the form's MainMenu to the Menu property
   Done
  
 Requirements:
   MenuItems in the MainMenu should be assigned TActions; these Actions should
   have Images.
   
 Features:
   For each menu, that has one (or more) items with Actions and Images, a bar
   is created (runtime). For each item in the menu with an Action and Image,
   a button is created. The bars can be switched on and of via a pop-up menu
   (right click the menubar). 
   
-------------------------------------------------------------------------------}
  
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, Menus;

type
  TMenuBar = class(TCustomControlBar)
  private
    { Private declarations }
    FMenu : TMainMenu;
    procedure PopupMenuClick(Sender: TObject);
  protected
    { Protected declarations }
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Build;
  published
    { Published declarations }
    property Menu: TMainMenu read FMenu write FMenu;
    property Align;
    property Anchors;
    property AutoDrag;
    property AutoSize;
    property BevelEdges;
    property BevelInner;
    property BevelOuter;
    property BevelKind;
    property BevelWidth;
    property BorderWidth;
    property Color;
    property Constraints;
    property Enabled;
    property ParentColor;
    property ParentFont;
    property ParentShowHint;
    property Picture;
    property PopupMenu;
    property RowSize;
    property RowSnap;
    property ShowHint;
    property TabOrder;
    property TabStop;
    property Visible;
    property OnCanResize;
    property OnClick;
    property OnConstrainedResize;
    property OnDblClick;
    property OnEnter;
    property OnExit;
    property OnGetSiteInfo;
    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnPaint;
    property OnResize;
  end;

procedure Register;

implementation

uses
  ComCtrls, Buttons, ToolWin, IniFiles;

function SkipSpecials(const sString:String): String;
var
  iPos : Integer;
  sResult : String;
begin
  sResult := '';
  for iPos := 1 to Length(sString) do
    if sString[iPos] in [' ', '-', '0'..'9', 'A'..'Z', 'a'..'z'] then
      sResult := sResult + sString[iPos];
  Result := sResult;
end;

procedure TMenuBar.PopupMenuClick(Sender: TObject);
var
  iIndex : Integer;
begin
  with (Sender as TMenuItem) do
    begin
      Checked := Not(Checked);
      iIndex := ControlCount - MenuIndex - 1;
      Controls[iIndex].Visible := Checked;
    end;
end;

procedure TMenuBar.Build;
var
  iMenu, iItem, i, iSize, iToolbar, iPos : Integer;
  doShow, hasActionAndImage, doSeparator, wasButton : Boolean;
  oToolbar : TToolbar;
  oPopup : TPopupMenu;
  oPopupMenuItem : TMenuItem;
  sTitle, sIniFile, sOptions, sOpt, sTemp, sHint : String;
  oIni : TIniFile;
begin
  if (FMenu = Nil) then
    Exit;
  for iToolbar := ControlCount-1 downto 0 do
    if (Controls[iToolbar] is TToolbar) then
      TToolbar(Controls[iToolbar]).Free;

  if (PopupMenu <> Nil) then
    PopupMenu.Free;
  doShow := False;
  sIniFile := ChangeFileExt(ParamStr(0),'.ini');
  oIni := TIniFile.Create(sIniFile);
  with FMenu do
    for iMenu := Items.Count-1 downto 0 do
      with Items[iMenu] do
        begin
          sTitle := SkipSpecials(Caption);
          sHint := Hint;
          hasActionAndImage := False;
          for iItem := 0 to Count-1 do
            if (Items[iItem].Action <> Nil) and (Items[iItem].ImageIndex >= 0) then
              hasActionAndImage := True;
          if (hasActionAndImage) then
            begin
              oToolbar := TToolbar.Create(Self);
              with oToolbar do
                begin
                  Parent      := Self;
                  Caption     := sTitle;
                  EdgeBorders := [];
                  EdgeInner   := esNone;
                  EdgeOuter   := esNone;
                  Flat        := True;
                  Hint        := sHint;
                  Images      := FMenu.Images;
                  Wrapable    := True;
                  iSize       := 1;
                  wasButton   := True;
                  for iItem := Count-1 downto 0 do
                    begin
                      doSeparator := False;
                      if (wasButton) then
                        for i := Count-1 downto iItem do
                          if (Items[i].ImageIndex >= 0) then
                            doSeparator := True;
                      if ((Items[iItem].Action <> Nil) and (Items[iItem].ImageIndex >= 0)) then
                        with TSpeedButton.Create(oToolbar) do
                          begin
                            Action    := Items[iItem].Action;
                            Parent    := oToolbar;
                            Caption   := '';
                            wasButton := True;
                            Inc(iSize, Width);
                          end
                      else
                        if ((Items[iItem].Caption = '-') and doSeparator) then
                          with TBevel.Create(oToolbar) do
                            begin
                              Parent    := oToolbar;
                              Shape     := bsSpacer;
                              Width     := 10;
                              wasButton := False;
                              Inc(iSize, Width);
                            end;
                    end;
                  Width  := iSize;
                  sOptions := oIni.ReadString('Toolbars', sTitle, '');
                  if (sOptions<>'') then
                    begin
                      sOpt    := sOptions;
                      iPos    := Pos(' ', sOpt);
                      sTemp   := Copy(sOpt, 1, iPos-1);
                      Left    := StrToInt(sTemp);
                      sOpt    := Copy(sOpt, iPos+1, MAXINT);
                      iPos    := Pos(' ', sOpt);
                      sTemp   := Copy(sOpt, 1, iPos-1);
                      Top     := StrToInt(sTemp);
                      sTemp   := Copy(sOpt, iPos+1, MAXINT);
                      Visible := (StrToInt(sTemp)=1);
                      if Visible then
                        doShow := True;
                    end;
                end;
            end;
        end;
  oIni.Free;
  oPopup := TPopupMenu.Create(Self);
  for iToolbar := ControlCount-1 downto 0 do
    if (Controls[iToolbar] is TToolbar) then
      with (Controls[iToolbar] as TToolbar) do
        begin
          oPopupMenuItem := TMenuItem.Create(Self);
          oPopupMenuItem.Caption := Caption;
          oPopupMenuItem.Checked := Visible;
          oPopupMenuItem.OnClick := PopupMenuClick;
          oPopup.Items.Add(oPopupMenuItem);
        end;
  PopupMenu := oPopup;
  if doShow then
    BevelEdges := [beLeft, beTop, beRight, beBottom]
  else
    BevelEdges := [beTop];
end;

constructor TMenuBar.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  AutoSize := True;
  Align    := alTop;
  Color    := clBtnFace;
  if (AOwner is TForm) then
    FMenu := TForm(AOwner).Menu;
end;

destructor TMenuBar.Destroy;
var
  oIni : TIniFile;
  sIniFile : String;
  iToolbar : Integer;
begin
  sIniFile := ChangeFileExt(ParamStr(0),'.ini');
  oIni := TIniFile.Create(sIniFile);
  for iToolbar := ControlCount-1 downto 0 do
    if (Controls[iToolbar] is TToolbar) then
      with (Controls[iToolbar] as TToolbar) do
        oIni.WriteString('Toolbars', Caption, Format('%d %d %d',[Left,Top,Ord(Visible)]));
  oIni.Free;
  if (PopupMenu <> Nil) then
    PopupMenu.Free;
  Inherited Destroy;
end;

procedure Register;
begin
  RegisterComponents('Samples', [TMenuBar]);
end;

end.
